<?php
/*
Plugin Name: Goodtill Stock Sync
Description: Sync your Goodtill POS products and stock quantities with WooCommerce.
Version: 1.1
Author: Goodtill
Author URI: https://thegoodtill.com
Plugin URI: https://thegoodtill.com/woocommerce-stock-sync/
WC requires at least: 3.3.0
WC tested up to: 4.0.1
*/

// Define some useful constants
define('GTCSYNC_NAME', 'Goodtill Stock Sync');
define('GTCSYNC_ID', 'gtc-stock-sync');
define('GTCSYNC_PREFIX', 'gtcsync_');
define('GTCSYNC_DIR', plugin_dir_path(__FILE__));
define('GTCSYNC_URL', gtc_sync_env('GTC_SYNC_URL', 'https://api.thegoodtill.com'));
define('GTCSYNC_APP', gtc_sync_env('GTC_SYNC_APP', 'https://pos.thegoodtill.com'));
define('GTCSYNC_IP', gtc_sync_env('GTC_SYNC_IP', '')); // Used for HTTP, add scheme
define('GTCSYNC_VERSION', '1.1');
define('GTCSYNC_ANALYTICS', true);
define('GTCSYNC_DATEFORMAT', 'd/m/Y H:i:s');
define('GTCSYNC_IMAGESETTINGS', gtc_sync_env('GTC_SYNC_IMAGESETTINGS', '240,80'));
define('GTCSYNC_DEBUG', gtc_sync_env('GTC_SYNC_DEBUG', false));
define('GTCSYNC_BUGSNAG_KEY', gtc_sync_env('GTC_SYNC_BUGSNAG_KEY', '6f38d1916dc015e98022327fbcb6918e'));
define('GTCSYNC_BUGSNAG_RELEASE_STAGE', GTCSYNC_DEBUG ? 'dev' : 'production');
define('GTCSYNC_PLUGIN_URL', plugin_dir_url(plugin_basename(__FILE__)));

// Load WP files
if (function_exists('get_plugins') == false)
    require_once ABSPATH.'wp-admin/includes/plugin.php';

// Load plugin files
require_once(GTCSYNC_DIR.'includes/base.php');
require_once(GTCSYNC_DIR.'includes/bugsnag.php');
require_once(GTCSYNC_DIR.'includes/client.php');
require_once(GTCSYNC_DIR.'includes/core.php');
require_once(GTCSYNC_DIR.'includes/cron.php');

// Load admin-specific files
if (is_admin()) {
    require_once(GTCSYNC_DIR.'includes/update.php');
    require_once(GTCSYNC_DIR.'includes/settings.php');
    require_once(GTCSYNC_DIR.'includes/import.php');
    require_once(GTCSYNC_DIR.'includes/export.php');
    require_once(GTCSYNC_DIR.'includes/admin.php');

    // Add our plugin settings link here, as it's easier to get the
    // plugin ID here due to the current file path
    $plugin = plugin_basename(__FILE__);
    add_filter('plugin_action_links_'.$plugin, function($links){
        $page = GTCSYNC_ID.'-settings';
        $settings_link = '<a href="options-general.php?page='.$page.'">'.__('Settings').'</a>';
        array_unshift($links, $settings_link);
        return $links;
    });

    // Load assets
    add_action('admin_enqueue_scripts', function($hook){

        // Enqueue our admin script on all pages, as this logic is used to inject elements
        // in to WooCommerce pages.
        wp_enqueue_script(
            GTCSYNC_PREFIX.'_admin',
            plugin_dir_url(__FILE__).'assets/js/admin.js',
            ['jquery'],
            GTCSYNC_VERSION,
            false
        );

        // Load our CSS, as some styles are applied to elements in WooCommerce pages.
        wp_enqueue_style(
            GTCSYNC_PREFIX.'_admin',
            plugin_dir_url(__FILE__).'assets/css/style.css'
        );
    });

    // Enqueue our admin CSS
    add_action('admin_enqueue_scripts', function(){
        wp_register_style('options_page_style', plugins_url('css/options_style.css',__FILE__));
        wp_enqueue_style('options_page_style');
    });

    // WooCommerce 3.6 requires us to load the REST API classes manually.
    add_action('admin_init', function() {
        $wc = WooCommerce::instance();
        if (version_compare($wc->version, '3.6.0') >= 0)
            $wc->api->rest_api_includes();
    });
}

// Activation, Deactivation and Uninstall Functions
// Update detection performed in includes/update.php
register_activation_hook(__FILE__, 'gtc_sync_activation');
register_deactivation_hook(__FILE__, 'gtc_sync_deactivation');

function gtc_sync_activation() {

    register_uninstall_hook(__FILE__, 'gtc_sync_uninstall');
}

function gtc_sync_deactivation() {

    // Remove WP Cron job
    GTCSyncCron::unregister();
}

function gtc_sync_uninstall(){

    // Remove all options. Would be more efficient to use
    // a query, but public API seems safer.
    foreach (wp_load_alloptions() as $option => $value) {
        if (strpos($option, GTCSYNC_PREFIX) === 0) {
            delete_option($option);
        }
    }
}

function gtc_sync_env($key, $default){

    $value = getenv($key);
    return empty($value) ? $default : $value;
}

function gtc_sync_get($key, $default=null){

    if (isset($_GET[$key]) == false)
        return $default;

    $value = $_GET[$key];

    return empty($value) ? $default : $value;
}