<?php

if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH.'wp-admin/includes/class-wp-list-table.php');
}

class GTCSyncAdmin extends GTCBase {

    protected $status_map;
    protected $page_id;

    public function __construct() {

        $this->page_id = GTCSYNC_ID.'-admin';

        $this->status_map = [
            'any' => 'All',
            'publish' => 'Published',
            'draft' => 'Draft',
            'pending' => 'Pending',
            'private' => 'Private',
        ];

        add_action('admin_menu', function(){
            add_submenu_page(
                'woocommerce',
                GTCSYNC_NAME,
                GTCSYNC_NAME,
                'manage_woocommerce',
                $this->page_id,
                [$this, 'adminPage']
            );
        });

        add_action(
            'admin_post_gtcsync_import',
            [$this, 'actionImport']
        );

        add_action(
            'admin_post_gtcsync_export',
            [$this, 'actionExport']
        );

        // Refresh the tab if exporting. Check that we're on the
        // export tab and there is an export queued.
        $exporter = new GTCSyncExport;
        $importer = new GTCSyncImport;

        $tab = gtc_sync_get('tab');
        $refresh = gtc_sync_get('page') == 'gtc-stock-sync-admin' &&
                    (($tab == 'export' && $exporter->isInProgress()) ||
                    (($tab == 'import' || empty($tab)) && $importer->isInProgress()));

        if ($refresh){
            // Set page to refresh in 3 seconds for next batch
            add_action('admin_head', function(){
                echo '<meta http-equiv="refresh" content="3">';
            });
        }
    }

    public function adminPage() {

        try {

           ?><div class="wrap">
                <h2><?php echo GTCSYNC_NAME; ?></h2>
           <?php

            $current_tab = gtc_sync_get('tab');

            $tabs = [
                ['id' => 'import', 'name' => 'Import'],
                ['id' => 'export', 'name' => 'Export'],
                ['id' => 'stock_status', 'name' => 'Stock Status'],
            ];

            if ($this->getClient()->hasCredentials() == false){
                ?>
                <div class="wrap">
                    <div class="error notice">
                        <p>Please input your credentials in the
                            <a href="options-general.php?page=gtc-stock-sync-settings">settings page</a>.
                        </p>
                    </div>
                </div>
                <?php
                return;
            }

            if ($this->hasTaxClasses() == false){
                ?>
                <div class="wrap">
                    <div class="error notice">
                        <p>Please map vat rates to tax classes on the
                            <a href="options-general.php?page=gtc-stock-sync-settings&tab=tax">settings page</a>.
                        </p>
                    </div>
                </div>
                <?php
                return;
            }

            ?><h2 class="nav-tab-wrapper">
                <?php foreach ($tabs as $index => $tab) { ?>
                    <a href="<?php echo esc_url(add_query_arg('tab', $tab['id'])); ?>" class="nav-tab
                        <?php if (($current_tab == '' && $index == 0) || $current_tab == $tab['id']) { echo 'nav-tab-active'; } ?>">
                        <?php echo $tab['name']; ?>
                    </a>
                <?php } ?>
            </h2>
            <div class="wrap">
            <?php

            if ($current_tab == $tabs[1]['id'])
                $this->tabExport();
            else if ($current_tab == $tabs[2]['id'])
                $this->tabStatus();
            else
                $this->tabImport();
            ?></div><?php

        } catch (Throwable $e) {

            ?>
            <div class="wrap">
                <div class="error notice">
                    <p><?php echo $e->getMessage(); ?></p>
                </div>
            </div>
            <?php
        }
    }

    public function tabImport(){

        $import = new GTCSyncImport;

        if ($import->isInProgress() == false){

            $products = null;
            try {
                $products = $import->getProducts();
            } catch (Throwable $e) {
                $e = new Exception('Failed to get products for import', 500, $e);
                $this->logError($e);
                ?>
                <div class="wrap">
                    <div class="error notice">
                        <p><?php echo $e->getMessage(); ?> - (<?php echo $e->getPrevious()->getMessage();?>)</p>
                    </div>
                </div>
                <?php
                return;
            }
            $import_issue_products = count(array_filter($products, function($p){ return $p['prevented_import']; }));
            $can_import = count(array_filter($products, function($p){ return $p['can_import']; }));

            $global_attributes = boolval(get_option(GTCSYNC_PREFIX.'global_attributes'));
            $global_attribute_msg = [
                true => 'Yes - Existing global attributes will be used.',
                false => 'No - Local attributes will be used.',
            ][$global_attributes];

            $table = new GTCImportProductListTable;
            $table->items = $products;
            $table->prepare_items();

                if ($import->getData()) {
                    $last_import_data = $import->getData();
                    $import->cancel();
                ?>
                    <div class="updated notice">
                        <p>Imported <?php echo count($last_import_data['products']); ?>
                            <a href="edit.php?post_type=product">products</a>.
                        </p>
                    </div>
                <?php } ?>
                <?php if ($can_import > 0) { ?>
                    <div class="updated notice"><p><?php echo $can_import; ?> products can be imported.</p></div>
                <?php } ?>
                <?php if ($import_issue_products > 0) { ?>
                    <div class="error notice"><p><?php echo $import_issue_products; ?> products cannot be imported -
                    please check the status column for each product.</p></div>
                <?php } ?>
                <form method="get" action="admin.php">
                    <input type="hidden" name="page" value="<?php echo $this->page_id; ?>">
                    <?php $table->search_box('Search', 'search'); ?>
                </form>
                <div id="container-table-product-import">
                <span>This page allows importing selected Goodtill products in to WooCommerce.</span>
                    <form method="post" action="admin-post.php">
                        <?php $table->display(); ?>
                        <input type="hidden" name="action" value="gtcsync_import" />
                        <?php wp_nonce_field('gtcsync_import_verify'); ?>

                        <?php if ($can_import > 0) { ?>
                            <p>The import process will import all products ticked in the list above.</p>
                            <table class="form-table">
                                <tr>
                                    <th scope="row"><label>Product status</label></th>
                                    <td>
                                        <select name="type">
                                            <option value="publish" selected="selected"><?php echo $this->status_map['publish']; ?></option>
                                            <option value="draft"><?php echo $this->status_map['draft']; ?></option>
                                            <option value="pending"><?php echo $this->status_map['pending']; ?></option>
                                            <option value="private"><?php echo $this->status_map['private']; ?></option>
                                        </select>
                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row"><label>Use global attributes</label></th>
                                    <td>
                                        <?php echo $global_attribute_msg; ?>
                                        <p class="description">See <a href="options-general.php?page=gtc-stock-sync-settings&tab=import_export">settings</a> for details.</p>
                                    </td>
                                </tr>
                            </table>
                            <?php submit_button('Import products', 'primary', 'import'); ?>
                        <?php } else { ?>
                            <p>No products are suitable for import.</p>
                        <?php } ?>
                    </form>
                </div>
            <?php
        } else {
            $data = $import->getData();
            ?><p>Import in progress - Please leave this tab open until the process is complete.</p>
            <p>Current product: <?php echo $data['index']; ?> of <?php echo count($data['products']); ?>
            <form method="post" action="admin-post.php">
                <input type="hidden" name="action" value="gtcsync_import" />
                <input type="hidden" name="cancel" value="true" />
                <?php wp_nonce_field('gtcsync_import_verify'); ?>
                <?php submit_button('Cancel import', 'primary', 'import'); ?>
            </form><?php
            $import->importProducts();
        }
    }

    public function tabExport(){

        ?>
        <span>This page allows exporting all WooCommerce products in a CSV ready to import in to Goodtill.</span>
        <?php

        $tomorrow = (new DateTime('tomorrow'))->format('Y-m-d');

        $exporter = new GTCSyncExport;
        if ($exporter->isInProgress()){
            $data = $exporter->getData();
            ?><p>Export in progress - Please leave this tab open until the process is complete.</p>
            <p>Current product: <?php echo $data['current_product'];?></p>
            <form method="post" action="admin-post.php">
                <input type="hidden" name="action" value="gtcsync_export" />
                <input type="hidden" name="cancel" value="true" />
                <?php wp_nonce_field('gtcsync_export_verify'); ?>
                <?php submit_button('Cancel export', 'primary', 'export'); ?>
            </form>
            <?php
            $exporter->exportProducts();
            return;
        }

        if ($exporter->hasData()) {
            $data = $exporter->getData();
            ?><form method="post" action="admin-post.php">
                <input type="hidden" name="action" value="gtcsync_export" />
                <?php wp_nonce_field('gtcsync_export_verify'); ?>
                <table class="form-table">
                    <tr>
                        <th scope="row"><label>Download CSV</label></th>
                        <td>
                            <?php
                                $date = $this->setTimeZone($data['updated']);
                                $type = isset($data['type']) ? strtolower($this->status_map[$data['type']]) : 'published';
                                $created_date_from = isset($data['created_date_from']) ? $data['created_date_from'] : null;
                                $created_date_to = isset($data['created_date_to']) ? $data['created_date_to'] : null;
                                $size = intval($data['size'] / 1024);

                                submit_button('Download product export', 'primary', 'download');

                                $date_range = '';
                                if (empty($created_date_from) == false)
                                    $date_range .= 'from '.DateTime::createFromFormat('Y-m-d', $created_date_from)->format('d/m/Y');

                                if (empty($created_date_to) == false)
                                    $date_range .= ' to '.DateTime::createFromFormat('Y-m-d', $created_date_to)->format('d/m/Y');

                                $str = sprintf('Export created on %s for %s products %s (%s kb).', $date, $type, $date_range, $size);
                            ?>
                            <p class="description">
                                <?php echo $str; ?>
                                <a href="<?php echo $exporter->getFileUrl(); ?>" title="This link can be used if the download button fails.">
                                    Alternate link.
                                </a>
                            </p>
                            <p class="description">The products CSV can be imported in the Goodtill Backoffice on the
                            <a href="<?php echo GTCSYNC_APP ?>/app/products/import">Product Import</a> page.</p>
                        </td>
                    </tr>
                </table>
            </form>
            <hr><?php
        }
        ?><form method="post" action="admin-post.php">
            <input type="hidden" name="action" value="gtcsync_export" />
            <?php wp_nonce_field('gtcsync_export_verify'); ?>
            <table class="form-table">
                <tr>
                    <th scope="row"><label>Product status filter</label></th>
                    <td>
                        <select name="type">
                            <option value="any"><?php printf('%s (%s)', $this->status_map['any'], $this->getProductCount()); ?></option>
                            <option value="publish" selected="selected"><?php printf('%s (%s)', $this->status_map['publish'], $this->getProductCount('publish')); ?></option>
                            <option value="draft"><?php printf('%s (%s)', $this->status_map['draft'], $this->getProductCount('draft')); ?></option>
                            <option value="pending"><?php printf('%s (%s)', $this->status_map['pending'], $this->getProductCount('pending')); ?></option>
                            <option value="private"><?php printf('%s (%s)', $this->status_map['private'], $this->getProductCount('private')); ?></option>
                        </select>
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label>Created date from filter</label></th>
                    <td>
                        <input type="date" name="created_date_from" max="<?php echo $tomorrow; ?>" placeholder="YYYY-MM-DD">
                        <p class="description">Only include products created after this date.</p>
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label>Created date to filter</label></th>
                    <td>
                        <input type="date" name="created_date_to" max="<?php echo $tomorrow; ?>" placeholder="YYYY-MM-DD">
                        <p class="description">Only include products create before this date.</p>
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label>Images</label></th>
                    <td>
                        <label>
                            <input type="checkbox" name="export_images" checked>
                            Enable
                        </label>
                        <p class="description">Export featured image for product / variant.<br>
                        Increases CSV size, may need to be disabled on some sites.</p>
                    </td>
                </tr>
                <tr>
                    <th scope="row"></th>
                    <td><?php submit_button('Start product export', 'primary', 'export'); ?></td>
                </tr>
            </table>

            <?php wp_nonce_field('gtcsync_export_verify'); ?>
        </form><?php
    }

    public function tabStatus(){

        // Get product data. Using a custom query is the only way to get this data quickly. The postmeta queries are
        // indexed so this query is very fast even with thousands of rows. wc_get_products etc all will perform
        // an order of magnitude more queries and is nowhere near fast enough to be useable.
        global $wpdb;
        $prefix = $wpdb->prefix;

        $query = "SELECT 
            {$prefix}posts.ID as 'product_id',
            {$prefix}posts.post_parent as 'parent_product_id',
            {$prefix}posts.post_title AS 'product_name',
            {$prefix}posts.post_excerpt AS 'product_attributes',
            count(product_variants.ID) as 'variant_count',
            {$prefix}posts.post_type AS 'product_type',
            postmeta_sku.meta_value as 'product_sku',
            postmeta_manage_stock.meta_value as 'manage_stock',
            postmeta_stock.meta_value as 'stock',
            postmeta_gtc_product_id.meta_value as 'gtc_product_id',
            postmeta_gtc_import_date.meta_value as 'gtc_import_date',
            postmeta_gtc_stock_update_success.meta_value as 'gtc_stock_update_success_date',
            postmeta_gtc_stock_update_failed.meta_value as 'gtc_stock_update_failed_date',
            postmeta_gtc_stock_update_reason.meta_value as 'gtc_stock_update_reason'
        FROM
            {$prefix}posts
        LEFT JOIN
            {$prefix}posts as product_variants on {$prefix}posts.ID = product_variants.post_parent and product_variants.post_status = 'publish'
        LEFT JOIN
            {$prefix}postmeta as postmeta_sku on {$prefix}posts.ID = postmeta_sku.post_id and postmeta_sku.meta_key = '_sku'
        LEFT JOIN
            {$prefix}postmeta as postmeta_manage_stock on {$prefix}posts.ID = postmeta_manage_stock.post_id and postmeta_manage_stock.meta_key = '_manage_stock'
        LEFT JOIN
            {$prefix}postmeta as postmeta_stock on {$prefix}posts.ID = postmeta_stock.post_id and postmeta_stock.meta_key = '_stock'
        LEFT JOIN
            {$prefix}postmeta as postmeta_gtc_product_id on {$prefix}posts.ID = postmeta_gtc_product_id.post_id and postmeta_gtc_product_id.meta_key = 'gtc-product-id'
        LEFT JOIN
            {$prefix}postmeta as postmeta_gtc_import_date on {$prefix}posts.ID = postmeta_gtc_import_date.post_id and postmeta_gtc_import_date.meta_key = 'gtc-import-date'
        LEFT JOIN
            {$prefix}postmeta as postmeta_gtc_stock_update_success on {$prefix}posts.ID = postmeta_gtc_stock_update_success.post_id and postmeta_gtc_stock_update_success.meta_key = 'gtc-stock-updated-date'
        LEFT JOIN
            {$prefix}postmeta as postmeta_gtc_stock_update_failed on {$prefix}posts.ID = postmeta_gtc_stock_update_failed.post_id and postmeta_gtc_stock_update_failed.meta_key = '_gtc-stock-update-failed-date'
        LEFT JOIN
            {$prefix}postmeta as postmeta_gtc_stock_update_reason on {$prefix}posts.ID = postmeta_gtc_stock_update_reason.post_id and postmeta_gtc_stock_update_reason.meta_key = '_gtc-stock-update-reason'
        WHERE
            {$prefix}posts.post_type IN ('product' , 'product_variation') AND
            {$prefix}posts.post_status = 'publish'
        GROUP BY
            {$prefix}posts.ID
        ORDER BY
            {$prefix}posts.ID ASC";

        $results = $wpdb->get_results($query, OBJECT);

        $count_success = 0;
        $count_error = 0;

        $stock_update_lines = [];
        foreach ($results as $result){

            // Stock values are sometimes incorrectly stored as floats, convert to int
            if ($result->stock !== null)
                $result->stock = intval($result->stock);

            $is_variant = $result->parent_product_id != 0;
            $is_parent = $result->variant_count > 0;

            // Get stock sync success and failure date times
            $update_success_str = $result->gtc_stock_update_success_date;
            $update_success_date = empty($update_success_str) ? null : DateTime::createFromFormat(GTCSYNC_DATEFORMAT, $update_success_str);

            $update_failed_str = $result->gtc_stock_update_failed_date;
            $update_failed_date = empty($update_failed_str) ? null : DateTime::createFromFormat(GTCSYNC_DATEFORMAT, $update_failed_str);

            $stock_sync_attempted = true;
            $stock_sync_date = null;
            $is_success = false;

            if ($update_success_date > $update_failed_date){
                $stock_sync_date = $update_success_date;
                $is_success = true;
            } else if ($update_success_date < $update_failed_date){
                $stock_sync_date = $update_failed_date;
                $is_success = false;
            } else {
                $stock_sync_attempted = false;
            }

            // Get update reason (either background sync or product view)
            $stock_reason_msg = '';
            $stock_reason = $result->gtc_stock_update_reason;
            if (empty($stock_reason) == false){
                if ($stock_reason == GTCSyncCore::STOCK_UPDATE_REASON_BACKGROUND)
                    $stock_reason_msg = 'background sync';
                else if ($stock_reason == GTCSyncCore::STOCK_UPDATE_REASON_VIEW)
                    $stock_reason_msg = 'product view';
            }

            // Status message
            $text = '';
            if ($stock_sync_attempted == false){
                $text = 'Stock sync not attempted';
            } else {
                $text = $is_success ? 'Succeeded' : 'Failed';
                $text .= ' '.human_time_diff($stock_sync_date->getTimestamp()).' ago';

                if ($stock_reason_msg)
                    $text .= ' ('.$stock_reason_msg.')';
            }

            // This URL is guessed instead of using get_edit_post_link() because that loads the post from
            // the DB and calling it in a loop produces far too many queries.
            $product_edit_url =  sprintf('/wp-admin/post.php?post=%d&action=edit', $result->product_id);

            // Get GTC edit link
            $gtc_product_edit_url = null;
            if (empty($result->gtc_product_id) == false)
                $gtc_product_edit_url = sprintf('%s/app/products/edit/%s', GTCSYNC_APP, $result->gtc_product_id);

            $stock_update_lines[] = [
                'product_id' => $result->product_id,
                'wc_product_url' => $product_edit_url,
                'gtc_product_url' => $gtc_product_edit_url,
                'is_variant' => $is_variant,
                'product_name' => $is_variant ? $result->product_attributes : $result->product_name,
                'product_sku' => $result->product_sku,
                'product_sku_html' => $is_parent ? '' : $this->skuHtml($result->product_sku),
                'stock_status_text' => $is_parent ? '' : $text,
                'stock_status_success' => $is_success,
                'manage_stock' => $is_parent ? '' : ucwords($result->manage_stock),
                'stock_quantity' => $is_parent ? '' : $result->stock,
            ];

            if ($is_parent == false){
                if ($is_success)
                    $count_success++;
                else
                    $count_error++;
            }
        }

        $table = new GTCStockStatusListTable();
        $table->items = $stock_update_lines;
        $table->prepare_items();

        ?>
            <div class="wrap">
                <div class="updated notice">
                    <?php
                    $entries = [];
                    if (get_option(GTCSYNC_PREFIX.'background_sync_enabled')){
                        $interval = intval(get_option(GTCSYNC_PREFIX . 'background_sync_interval_seconds')) / 60;
                        $entries[] = sprintf('Stock synced in the background every %d minutes.', $interval);
                    }

                    if (get_option(GTCSYNC_PREFIX.'sync_enabled'))
                        $entries[] = 'Stock synced as customers view products.';

                    if (empty($entries)){
                        print('Stock syncing disabled');
                    } else {
                        print('<ul class="gtc-box-list">');
                        foreach ($entries as $entry)
                            printf('<li>%s</li>', $entry);
                        print('</ul>');
                    }
                    ?>
                </div>
                <?php if ($count_success > 0) { ?>
                    <div class="updated notice"><p><?php echo $count_success; ?> products/variants are syncing stock levels correctly.</p></div>
                <?php } ?>
                <?php if ($count_error > 0) { ?>
                    <div class="error notice"><p><?php echo $count_error; ?> products/variants arenot syncing stock levels, this may be due to:</p>
                        <ul class="gtc-box-list">
                            <li>Products and variants are missing SKUs.</li>
                            <li>Products and variants do not have unique SKUs.</li>
                            <li>The SKUs for products and variants are not set the same in both Goodtill and WooCommerce.</li>
                        </ul>
                    </div>
                <?php } ?>
            </div>
            <div id="container-table-sync-status">
                <span>The table below displays all published products, including their variants, and whether the stock level
                is currently syncing with Goodtill.</span>
                <?php $table->display(); ?>
            </div>
        <?php
    }

    public function actionImport() {

        check_admin_referer('gtcsync_import_verify');

        $import = new GTCSyncImport;

        // Cancel running import
        if (isset($_POST['import']) && isset($_POST['cancel'])){
            $import->cancel();
        }
        // Begin import
        else if (isset($_POST['import']) && isset($_POST['import_product'])){

            // Get products
            $product_ids = $_POST['import_product'];
            $status = $_POST['type'];

            $import->importProducts($product_ids, $status);
        }

        wp_redirect(admin_url('admin.php?page='.$this->page_id));
    }

    public function actionExport() {

        check_admin_referer('gtcsync_export_verify');

        $export = new GTCSyncExport;

        // Cancel running import
        if (isset($_POST['export']) && isset($_POST['cancel'])){
            $export->cancel();
        }
        // Begin export
        else if (isset($_POST['export'])){

            // Delete previous data
            if ($export->hasData())
                $export->cancel();

            // Get params
            $type = $_POST['type'];
            $export_images = empty($_POST['export_images']) == false;
            $created_date_from = $_POST['created_date_from'];
            $created_date_to = $_POST['created_date_to'];

            $export->exportProducts($type, $export_images, $created_date_from, $created_date_to);
        }
        // Download exported CSV
        else if (isset($_POST['download'])){

            $data = $export->getData();
            $date = $this->setTimeZone($data['updated'], 'Y-m-d_H-i-s');
            $type = isset($data['type']) ? strtolower($this->status_map[$data['type']]) : 'published';

            $fname = sprintf(
                'wc_to_gtc_products_%s_%s_%s.csv',
                sanitize_title(get_bloginfo('name')),
                $type,
                $date
            );

            // Set headers
            header('Content-Type: text/csv');
            header('Content-Disposition: attachment; filename="'.$fname.'"');

            // Stream file
            $path = $export->getFilePath();
            $fd = fopen($path,'r');
            fpassthru($fd);

            die();
        }

        wp_redirect(admin_url('admin.php?page='.$this->page_id.'&tab=export'));
    }
}

class GTCStockStatusListTable extends WP_List_Table {

    function __construct() {

        parent::__construct([
            'singular' => 'Product',
            'plural' => 'Products',
            'ajax' => false
        ]);

        $this->_column_headers = [
            $this->get_columns(),
            [],
            $this->get_sortable_columns()
        ];
    }

    function get_columns(){
        return [
            'product_name' => 'Product',
            'product_sku' => 'SKU',
            'manage_stock' => 'Stock managed',
            'stock_quantity' => 'Stock quantity',
            'stock_status_text' => 'Stock sync status',
        ];
    }

    function get_sortable_columns(){
        return [

        ];
    }

    function column_default($item, $column_name) {

        $val = @$item[$column_name];
        $content = $val;

        if ($column_name == 'stock_status_text'){
            $content = sprintf('<span style="color: %s">%s</span>', $item['stock_status_success'] ? 'green' : 'red', $val);
        } else if ($column_name == 'product_name' && $item['is_variant'] == false){
            $content = sprintf('<a href="%s" title="View in WooCommerce">%s</a>', $item['wc_product_url'], $val);
        }

        if ($column_name == 'product_name'){

            if ($item['is_variant'] == false && empty($item['gtc_product_url']) == false){
                $content = sprintf(
                    '<a href="%s" target="_blank" title="View in Goodtill"><img class="gtc-icon" src="%sassets/img/icon.png"></a> %s',
                    $item['gtc_product_url'],
                    GTCSYNC_PLUGIN_URL,
                    $content
                );
            }

            if ($item['is_variant'])
                $content = sprintf('<span style="padding-left: 20px;">%s</span>', $content);
        }

        if ($column_name == 'product_sku')
            return $item['product_sku_html'];

        return $content;
    }

    function prepare_items(){

    }
}

class GTCImportProductListTable extends WP_List_Table {

    function __construct() {

        parent::__construct([
            'singular' => 'Product',
            'plural' => 'Products',
            'ajax' => false
        ]);

        $this->_column_headers = [
            $this->get_columns(),
            [],
            $this->get_sortable_columns()
        ];
    }

    function get_columns(){

        return [
            'cb' => '<input type="checkbox"/>',
            'product_name' => 'Product',
            'category' => 'Category',
            'product_sku' => 'SKU',
            'variant_count' => 'Variants',
            'status' => 'Status'
        ];
    }

    function get_sortable_columns(){
        return [
            'product_name' => ['product_name', false],
            'category' => ['category', false],
            'product_sku' => ['product_sku', false],
            'variant_count' => ['variant_count', false],
            'status' => ['status', false]
        ];
    }

    function column_default($item, $column_name) {

        $val = htmlspecialchars(@$item[$column_name]);
        $content = $val;

        if ($column_name == 'product_name'){
            if (isset($item['woo_product_id'])){
                // This URL is guessed instead of using get_edit_post_link() because that loads the post from
                // the DB and calling it in a loop produces far too many queries.
                $product_edit_url =  sprintf('/wp-admin/post.php?post=%d&action=edit', $item['woo_product_id']);
                $content = sprintf('<a href="%s" title="View in WooCommerce">%s</a>', $product_edit_url, $val);
            }

            return sprintf(
                '<a href="%s/app/products/edit/%s" target="_blank" title="View in Goodtill"><img class="gtc-icon" src="%sassets/img/icon.png"></a> %s',
                GTCSYNC_APP,
                $item['product_id'],
                GTCSYNC_PLUGIN_URL,
                $content
            );
        }

        if ($column_name == 'product_sku')
            return $item['product_sku_html'];

        return $content;
    }

    function prepare_items(){

        $items = $this->items;

        // Search
        $search = isset($_REQUEST['s']) ? $_REQUEST['s'] : false;
        if ($search){
            $search = strtolower($search);

            $items = array_filter($items, function($item) use ($search){
                return strpos(strtolower($item['product_name']), $search) !== false ||
                    strpos(strtolower($item['category']), $search) !== false ||
                    strtolower($item['product_sku']) == $search;
            });
        }

        // Order
        $order_by = !empty($_REQUEST['orderby']) ? wp_unslash($_REQUEST['orderby']) : 'status';
        $order = !empty($_REQUEST['order']) ? wp_unslash($_REQUEST['order']) : 'asc';

        usort($items, function($a, $b) use ($order_by, $order){
            // Standard sort
            $result = strnatcmp($a[$order_by], $b[$order_by]);

            // ASC/DESC
            if ($order != 'asc')
                $result *= -1;

            return $result;
        });

        $this->items = $items;
    }

    function column_cb($item) {
        return sprintf(
            '<input type="checkbox" name="import_product[]" value="%s" %s>',
            $item['product_id'],
            $item['can_import'] ? '' : 'disabled'
        );
    }
 }

$GLOBALS['gtcsync_admin'] = new GTCSyncAdmin();