<?php

class GTCBugsnag extends GTCBase {

    protected $api_key;

    public function __construct($api_key)
    {
        $this->api_key = $api_key;
    }

    public function notify(Throwable $exception, $metadata = [], $user = [],
                           $severity = 'error', $release_stage = 'production')
    {
        $exceptions = [];
        $current_exception = $exception;
        while ($current_exception != null){
            $ex_arr = [];
            $ex_arr['errorClass'] = get_class($current_exception);
            $ex_arr['message'] = $current_exception->getMessage();
            $ex_arr['stacktrace'] = [
                [
                    'file' => $current_exception->getFile(),
                    'lineNumber' => $current_exception->getLine() - 1,
                ]
            ];

            try {
                $line_count = 3;
                $file = file_get_contents($current_exception->getFile());
                $lines = explode("\n", $file);
                $index = $current_exception->getLine() - 1;
                $start_index = max($index - $line_count, 0);
                $end_index = min($index + $line_count, count($lines) - 1);

                $code = [];
                foreach (range($start_index, $end_index) as $line_no)
                    $code[$line_no] = $lines[$line_no];

                $ex_arr['stacktrace'][0]['code'] = $code;
            } catch (Throwable $e){}

            $exceptions[] = $ex_arr;

            $current_exception = $current_exception->getPrevious();
        }

        $request = [];
        try {
            $request = [
                'clientIp' => $_SERVER['REMOTE_ADDR'],
                'httpMethod' => $_SERVER['REQUEST_METHOD'],
                'url' => $_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'],
            ];
        } catch (Throwable $e) {
            // Probably CLi
        }

        $data = [
            'apiKey' => $this->api_key,
            'events' => [
                [
                    'app' => [
                        'releaseStage' => $release_stage,
                        'version' => GTCSYNC_VERSION,
                    ],
                    'breadcrumbs' => [],
                    'context' => get_class($exception),
                    'device' => $this->getSystemInfo(),
                    'exceptions' => $exceptions,
                    'groupingHash' => $exception->getMessage(),
                    'metaData' => $metadata,
                    'payloadVersion' => '3',
                    'severity' => $severity,
                    'threads' => [],
                    'user' => $user,
                    'request' => $request,
                ]
            ],
            'notifier' => [
                'name' => 'Goodtill - Woocommerce Stock Sync Plugin',
                'url' => 'https://thegoodtill.com/woocommerce-stock-sync',
                'version' => GTCSYNC_VERSION,
            ]
        ];

        $endpoint = 'https://notify.bugsnag.com';

        try {
            $response = wp_remote_post(
                $endpoint,
                [
                    'method' => 'POST',
                    'timeout' => 5,
                    'redirection' => 5,
                    'httpversion' => '1.1',
                    'blocking' => false,
                    'headers' => [],
                    'body' => json_encode($data),
                    'cookies' => [],
                    'sslverify' => true,
                ]
            );

            $this->handleWpError($response);
        } catch (Throwable $e) {
            $this->getLogger()->error('Failed to send error report to bugsnag');
        }
    }
}