<?php

class GTCSyncUpdater extends GTCBase {

    CONST VERSION_KEY = GTCSYNC_PREFIX.'version';

    public function __construct(){

        add_action('admin_init', function() {
            $this->checkVersionChange();
        });
    }

    private function checkVersionChange(){

        $old_version = get_option(static::VERSION_KEY);
        $new_version = GTCSYNC_VERSION;

        $version_compare = version_compare($old_version, $new_version);

        if (empty($old_version)) {
            $this->update(null, $new_version);
        } else if ($version_compare === -1) {
            $this->update($old_version, $new_version);
        } else if ($version_compare === 1) {
            $this->logError(new Exception('Plugin downgraded'), ['old_version' => $old_version, 'new_version' => $new_version]);
            $this->update($old_version, $new_version);
        }
    }

    private function update($old_version, $new_version){

        // Record new version
        update_option(static::VERSION_KEY, $new_version);

        // Log update
        $has_previous_version = empty($old_version) == false;
        $old_version_str = $has_previous_version == false ? 'unknown' : $old_version;
        $this->getLogger()->info('Plugin upgraded from '.$old_version_str.' to '.$new_version);

        // Stop here if no old version
        if ($has_previous_version == false)
            return;

        // Prevent resuming import update, as format of serialised temp data may have changed
        if (delete_option(GTCSYNC_PREFIX.'import_temp'))
            $this->getLogger()->info('Deleted import temp data');

        // Prevent resuming export update, as format of serialised temp data may have changed
        if (delete_option(GTCSYNC_PREFIX.'export_temp'))
            $this->getLogger()->info('Deleted export temp data');
    }
}

$GLOBALS['gtcsync_update'] = new GTCSyncUpdater();